<?php

namespace App\Http\Controllers;

use App\Models\File;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use App\Http\Controllers\FileGestionController;
use Exception;
use Illuminate\Support\Facades\Gate;

use function PHPUnit\Framework\isNull;

class FileController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        return view('files.home', [
            'files' => File::with('creator')->latest()->get(),
        ]);
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        // Go to the crreation form
        return view('files.create');
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        // All the validations
        $validator = Validator::make(
            $request->all(),
            [
                'name' => 'required|string|max:63',
                'description' => 'required|string|max:1023',
                'file' => 'required',
            ],
            [
                'file.required' => 'Please select a file.',
            ]
        );

        // If something wrong, we return on the form with errors
        if ($validator->fails()) {
            return redirect(route('files.create'))
                ->withErrors($validator)
                ->withInput();
        }

        // Get the file infos
        $file = $request->file('file');
        $file_name = $file->getClientOriginalName();

        // Create the array for the eloquent relation
        $validate = [
            'name' => $request->name,
            'description' => $request->description,
            'link' => str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT']) . "/../resources/files/encrypted_" . $file_name
        ];

        // Encrypt and store the file
        $validator->after(function ($validator) use ($file) {

            //The path where all the files are stored
            $path = str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT']) . "/../resources/files";

            $encrypted_info = app(FileGestionController::class)->file_encrypt($file);

            // Save infos in a file for future decryption
            if (!($encrypted_info_file = fopen($path . "/eif.txt", "a+"))) {
                $validator->errors()->add(
                    'file',
                    'Unable to save the file\'s data.'
                );
            }

            //Save the infos
            fwrite($encrypted_info_file, $encrypted_info[0] . "," . $encrypted_info[1] . "\n");

            //Close the file
            fclose($encrypted_info_file);
        });

        // If something wrong, we return on the form with errors
        if ($validator->fails()) {
            return redirect(route('files.create'))
                ->withErrors($validator)
                ->withInput();
        }

        // Create the eloquent relation
        $request->user()->files()->create($validate);

        // Redirect to home
        return redirect(route('files.index'));
    }

    /**
     * Display the specified resource.
     */
    public function show(file $file)
    {
        return view('files.show', [
            'file' => $file,
        ]);
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(file $file)
    {
        Gate::authorize('update', $file);

        return view('files.edit', [
            'file' => $file,
        ]);
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, file $file)
    {
        Gate::authorize('update', $file);

        // All the validations
        $validator = Validator::make(
            $request->all(),
            [
                'name' => 'required|string|max:63',
                'description' => 'required|string|max:1023',
            ]
        );

        // If something wrong, we return on the form with errors
        if ($validator->fails()) {
            return redirect(route('files.edit', $file))
                ->withErrors($validator)
                ->withInput();
        }

        // Create the array for the eloquent relation
        $validate = [
            'name' => $request->name,
            'description' => $request->description,
        ];

        if ($request->hasFile('file')) {
            // Get the file infos
            $new_file = $request->file('file');
            $new_file_name = $new_file->getClientOriginalName();

            // Create the array for the eloquent relation
            $validate['link'] = str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT']) . "/../resources/files/encrypted_" . $new_file_name;

            // Encrypt and store the file
            $validator->after(function ($validator) use ($new_file) {

                //The path where all the files are stored
                $path = str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT']) . "/../resources/files";

                $encrypted_info = app(FileGestionController::class)->file_encrypt($new_file);

                // Save infos in a file for future decryption
                if (!($encrypted_info_file = fopen($path . "/eif.txt", "a+"))) {
                    $validator->errors()->add(
                        'file',
                        'Unable to save the file\'s data.'
                    );
                }

                //Save the infos
                fwrite($encrypted_info_file, $encrypted_info[0] . "," . $encrypted_info[1] . "\n");

                //Close the file
                fclose($encrypted_info_file);
            });

            // If something wrong, we return on the form with errors
            if ($validator->fails()) {
                return redirect(route('files.edit', $file))
                    ->withErrors($validator)
                    ->withInput();
            }
        }

        // Create the eloquent relation
        $request->user()->files()->update($validate);

        // Redirect to home
        return redirect(route('files.index'));
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Request $request, file $file)
    {
        Gate::authorize('delete', $file);

        // All the validations
        $validator = Validator::make(
            $request->all(),
            []
        );

        // If something wrong, we return on the form with errors
        $validator->validate();

        $validator->after(function ($validator) use ($file, $request) {
            $path = $file->link;

            // Check if the file exist
            if (!file_exists($path)) {
                $validator->errors()->add(
                    'file',
                    'Cannot find the file ' . $path
                );
            }

            if (!($encrypted_info_file = fopen(str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT']) . "/../resources/files/eif.txt", "a+"))) {
                $validator->errors()->add(
                    'file',
                    'Unable to open the data of the file'
                );
            }

            //Read the infos
            $infos = fread($encrypted_info_file, filesize(str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT']) . "/../resources/files/eif.txt"));

            // Close the file
            fclose($encrypted_info_file);

            // Let's find the matching line
            $pattern = "/" . str_replace('/', "\/", $path) . ",(.*)/";
            if (!(preg_match_all($pattern, $infos, $matches))) {
                $validator->errors()->add(
                    'file',
                    'Unable to find the file\'s data'
                );
            }

            // Delete the line
            $contents = str_replace($matches[0][0] . "\n", '', $infos);

            //Reopen the file to rewite it
            $encrypted_info_file = fopen(str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT']) . "/../resources/files/eif.txt", "w+");

            // Rewrite the file
            fwrite($encrypted_info_file, $contents);

            // Close the file
            fclose($encrypted_info_file);

            // Delete the file
            unlink($path);
        });

        // If something wrong, we return on the form with errors
        $validator->validate();

        $file->delete();

        return redirect(route('files.index'));
    }

    /**
     * Download form
     */
    public function download_form(int $id)
    {
        $file = File::find($id);

        return view('files.download', [
            'file' => $file,
        ]);
    }

    /**
     * Download the file
     */
    public function download(Request $request, int $id)
    {
        $file = File::find($id);

        // All the validations
        $validator = Validator::make(
            $request->all(),
            [
                'myfolder' => 'required|string|max:1023',
            ]
        );

        // If something wrong, we return on the form with errors
        $validator->validate();

        $validator->after(function ($validator) use ($file, $request) {
            //The path where all the files are stored
            $path = str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT']) . "/../resources/files";

            //if (!isNull($file)) {
            // Get parameters
            //get the file name
            $path_array = explode("/", $file->link);
            $file_name = $path_array[array_key_last($path_array)];

            if (!($encrypted_info_file = fopen($path . "/eif.txt", "a+"))) {
                $validator->errors()->add(
                    'myfolder',
                    'Unable to open the data of the file'
                );
            }

            //Read the infos
            $infos = fread($encrypted_info_file, filesize($path . "/eif.txt"));

            $pattern = "/" . str_replace('/', "\/", $file->link) . ",(.*)/";

            if (!(preg_match_all($pattern, $infos, $matches))) {
                $validator->errors()->add(
                    'myfolder',
                    'Unable to find the file\'s data'
                );
            }
            /*$validator->errors()->add(
                'myfolder',
                /*'Infos: ' . var_dump($infos) . '\nPattern: ' . var_dump($pattern) . '\nMatches: ' .
                var_dump($matches)
            );

            exit();*/
            $tmp_file = app(FileGestionController::class)->file_decrypt(array($file->link, $matches[1][0]));

            //Close the file
            fclose($encrypted_info_file);

            //We want to retrive the file name without the "encrypted_" before
            $pattern = "/encrypted_(.*)/";
            preg_match_all($pattern, $file_name, $matches);

            try {
                if (!copy($tmp_file, $request->myfolder . "/" . $matches[1][0])) {
                    $validator->errors()->add(
                        'myfolder',
                        'Impossible to download the file'
                    );
                }
            } catch (Exception $ex) {
                $validator->errors()->add(
                    'myfolder',
                    $ex->getMessage()
                );
            }
            /*} else {
                $validator->errors()->add(
                    'myfolder',
                    'No file to upload!'
                );
            }*/
        });

        // If something wrong, we return on the form with errors
        $validator->validate();

        return redirect()->route('files.show', [
            'file' => $file,
        ]);
    }
}
